#include <avr/io.h>
#include <avr/eeprom.h>
#include "button.h"
#include "rtc.h"
#include "ir.h"
#include "usart.h"
#include "onewire.h"
#include "ds18B20.h"
#include "app_clk.h"
#include "app_tmp.h"
#include "app_irt.h"
#include "app_urt.h"
#include "app.h"

static APP_NAME EEMEM ee_app = APP_CLK;
static uint16_t EEMEM ee_clk_code = 0;
static uint16_t EEMEM ee_tmp_code = 0;
static uint16_t EEMEM ee_urt_code = 0;
static uint16_t clk_code = 0;
static uint16_t tmp_code = 0;
static uint16_t urt_code = 0;

static void APP_LoadIrCodes(void) {
    clk_code = eeprom_read_word((const uint16_t*) &ee_clk_code);
    tmp_code = eeprom_read_word((const uint16_t*) &ee_tmp_code);
    urt_code = eeprom_read_word((const uint16_t*) &ee_urt_code);
}

void APP_Init(void) {
    APP_LoadIrCodes();
    APP_NAME app = eeprom_read_byte((const uint8_t*) &ee_app);
    APP_Run(app);
}

void APP_SaveIrCode(APP_NAME app, uint16_t code) {
    switch(app) {
        case APP_CLK: eeprom_update_word((uint16_t*)&ee_clk_code, code); break;
        case APP_TMP: eeprom_update_word((uint16_t*)&ee_tmp_code, code); break;
        case APP_URT: eeprom_update_word((uint16_t*)&ee_urt_code, code); break;
        default: break;
    }
    APP_LoadIrCodes();
}

void APP_Default(APP_NAME app) {
    eeprom_update_byte((uint8_t*)&ee_app, app);
}

void APP_PacketReceived(IR_PACKET packet) {
    uint16_t code = (packet.address<<8) | packet.command;
    if(code == clk_code) {
        APP_Run(APP_CLK);
    } else if(code == tmp_code) {
        APP_Run(APP_TMP);
    } else if(code == urt_code) {
        APP_Run(APP_URT);
    }
}

void APP_Run(APP_NAME app) {
    BUTTON_Init();
    RTC_Init();
    IR_Init();
    IR_PacketReceived = APP_PacketReceived;
    USART_Init();
    ONEWIRE_Init();
    DS18B20_Init();
    switch(app) {
        default:
        case APP_CLK: CLK_Init(); break;
        case APP_TMP: TMP_Init(); break;
        case APP_URT: URT_Init(); break;
        case APP_IRT: IRT_Init(); break;
    }
}
