#include <avr/io.h>
#include <avr/interrupt.h>
#include "ATmega88PA.h"
#include "device.h"
#include "display.h"
#include "stub.h"
#include "button.h"

static struct {
    uint8_t state;
    uint8_t changed;
} button = {0x00, 0x00};

static struct {
    uint8_t low;
    uint8_t high;
} vcount = {0xFF, 0xFF};

static void BUTTON_Debounce(void) {
    button.changed = ((BUTTON_RAW_STATE) ^ button.state);
    // Decrease vertical counters where state_changed has a bit set (1)
    // and set the others to 11 (reset state)
    vcount.low = ~(vcount.low & button.changed);
    vcount.high = vcount.low ^ (vcount.high & button.changed);
    // Update state_changed to have a 1 only if the counter overflowed
    button.changed &= vcount.high & vcount.low;
    // Change button_state for the buttons who’s counters rolled over
    button.state ^= button.changed;
    // Update button_down with buttons who’s counters rolled over
    // and who’s state is 1 (pressed)
    if(button.state & button.changed) {
        BUTTON_Pressed(button.state);
    }
    if(~button.state & button.changed) {
        BUTTON_Released(button.changed);
    }
}

void BUTTON_Activate(void) {
    TIMER0_CLEAR_COMPARE_MATCH_B_INTERRUPT_FLAG;
    TIMER0_COMPARE_MATCH_B_INTERRUPT_ENABLE;
}

static void BUTTON_Deactivate(void) {
    TIMER0_COMPARE_MATCH_B_INTERRUPT_DISABLE;
}

void BUTTON_Init(void) {
    DISPLAY_ButtonBacklight(0);
    BUTTON_Pressed = STUB_uint8;
    BUTTON_Released = STUB_uint8;
    BUTTON_Deactivate();
}

static uint8_t counter = 0;

ISR(TIMER0_COMPB_vect) {
    if(++counter%40 == 0) {
        BUTTON_Debounce();
    }
}
