#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/power.h>
#include <avr/wdt.h>
#include "ATmega88PA.h"
#include "osc.h"
#include "device.h"
#include "display.h"
#include "app.h"

int main(void) {
    // Enable watchdog (4s)
    wdt_enable(WDTO_4S);
    // Internal RC oscillator at 8 MHz
    clock_prescale_set(clock_div_1);
    // Internal RC oscillator calibration to 7,3728 MHz
    OSC_Calibrate();
    // Select default application
    switch(BUTTON_RAW_STATE) {
        case SW1: APP_Default(APP_CLK); break;
        case SW2: APP_Default(APP_TMP); break;
        case SW3: APP_Default(APP_URT); break;
        case SW4: APP_Default(APP_IRT); break;
    }
    // Initialization
    DEVICE_Init();
    DISPLAY_Init();
    APP_Init();
    ENABLE_INTERRUPTS;
    // main loop
    while(1) {
        if(POWER_OFF) {
            DISPLAY_SetBrightness(0);
            DEVICE_Sleep();
        }
    }
}
